# SPDX-FileCopyrightText: 2023-present adamws <adamws@users.noreply.github.com>
#
# SPDX-License-Identifier: MIT
from hatchling.plugin.manager import PluginManager

from hatch_kicad.build import KicadBuilder


def test_hooks():
    plugin_manager = PluginManager()
    assert plugin_manager.builder.get("kicad-package") is KicadBuilder



# SPDX-FileCopyrightText: 2023-present adamws <adamws@users.noreply.github.com>
#
# SPDX-License-Identifier: MIT
from __future__ import annotations

from pathlib import Path
from typing import Any, TypedDict

from hatchling.builders.config import BuilderConfig


class Person(TypedDict):
    name: str
    contact: dict[str, str]


class KicadBuilderConfig(BuilderConfig):
    _BASE = "tool.hatch.build.targets.kicad-package"

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)

        self.__name: str | None = None
        self.__description: str | None = None
        self.__description_full: str | None = None
        self.__identifier: str | None = None
        self.__author: Person | None = None
        self.__maintainer: Person | None = None
        self.__license: str | None = None
        self.__resources: dict[Any, Any] | None = None
        self.__status: str | None = None
        self.__kicad_version: str | None = None
        self.__kicad_version_max: str | None = None
        self.__tags: list[str] | None = None
        self.__icon: Path | None = None

    def required_str(self, name: str) -> str:
        if name in self.target_config:
            value = self.target_config[name]
            if not isinstance(value, str) and not (
                isinstance(value, list)
                and len(value)
                and all(isinstance(item, str) for item in value)
            ):
                msg = f"Field `{self._BASE}.{name}` must be a string or list of strings"
                raise TypeError(msg)
        else:
            msg = f"Field `{self._BASE}.{name}` not found"
            raise TypeError(msg)
        if isinstance(value, list):
            value = "".join(value)
        return value

    @property
    def name(self) -> str:
        """
        The human-readable name of the package
        """
        if not self.__name:
            self.__name = self.required_str("name")
        return self.__name

    @property
    def description(self) -> str:
        """
        A short free-form description of the package that will be shown
        in the PCM alongside the package name.
        May contain a maximum of 150 characters.
        """
        if not self.__description:
            self.__description = self.required_str("description")
        return self.__description

    @property
    def description_full(self) -> str:
        """
        A long free-form description of the package that will be shown
        in the PCM when the package is selected by the user.
        May be a string or list of strings with included line breaks.
        """
        if not self.__description_full:
            self.__description_full = self.required_str("description_full")
        return self.__description_full

    @property
    def identifier(self) -> str:
        """
        The unique identifier for the package.
        May contain only alphanumeric characters and the dash (-) symbol.
        Must be between 2 and 50 characters in length.
        Must start with a latin character and end with a latin character or a numeral.
        """
        if not self.__identifier:
            self.__identifier = self.required_str("identifier")
        return self.__identifier

    @property
    def type(self) -> str:
        return "plugin"

    def get_person(self, name: str) -> Person | None:
        if name in self.target_config:
            person = self.target_config[name]
            if not isinstance(person, dict):
                msg = f"Field `{self._BASE}.{name}` must be a dictionary"
                raise TypeError(msg)
            if "name" not in person:
                msg = f"Field `{self._BASE}.{name}` must have `name` property"
                raise TypeError(msg)
            name = person["name"]
            contact = person
            del contact["name"]
            return {"name": name, "contact": contact}
        return None

    @property
    def author(self) -> Person:
        """
        Object containing one mandatory field, `name`, containing the name
        of the package creator.
        An optional `contact` field may be present,
        containing free-form fields with contact information.
        """
        if not self.__author:
            author: Person | None = self.get_person("author")
            if not author:
                authors: list[Any] = self.builder.metadata.core.authors
                if authors and "name" in authors[0]:
                    name = authors[0]["name"]
                    contact = {}
                    if email := authors[0].get("email", {}):
                        contact = {"email": email}
                    author = Person(name=name, contact=contact)
                else:
                    msg = (
                        f"Field `{self._BASE}.author` not found, "
                        "failed to get author from `project.authors` value"
                    )
                    raise TypeError(msg)
            self.__author = author
        return self.__author

    @property
    def maintainer(self) -> Person | None:
        """
        Semantics same as `author`, but containing information
        about the maintainer of the package
        """
        if not self.__maintainer:
            maintainer: Person | None = self.get_person("maintainer")
            if not maintainer:
                maintainers: list[Any] = self.builder.metadata.core.maintainers
                if maintainers and "name" in maintainers[0]:
                    maintainer = {
                        "name": maintainers[0]["name"],
                        "contact": {"email": maintainers[0].get("email", "-")},
                    }
                else:
                    maintainer = None
            self.__maintainer = maintainer
        return self.__maintainer

    @property
    def license(self) -> str:
        if not self.__license:
            # if `self.config` does not contain `license`,
            # try to deduce it from project settings
            if "license" in self.target_config:
                _license = self.required_str("license")
            elif not (_license := self.builder.metadata.core.license):
                msg = (
                    "Field `tool.hatch.build.targets.kicad-package.license` not found, "
                    "failed to deduce license from `project.license` value.\n"
                    'Define `license = {text = "<value>"} in `project` or '
                    '`license = "<value>" in `tool.hatch.build.targets.kicad-package'
                )
                raise TypeError(msg)
            self.__license = _license
        return self.__license

    @property
    def resources(self) -> dict[Any, Any]:
        if not self.__resources:
            if "resources" in self.target_config:
                resources: dict[Any, Any] = self.target_config["resources"]
                if not isinstance(resources, dict):
                    msg = f"Field `{self._BASE}.resources` must be a dictionary"
                    raise TypeError(msg)
            else:
                # `resources` are not mandatory so if can't be found nothing happens
                resources = self.builder.metadata.core.urls
            self.__resources = resources
        return self.__resources

    @property
    def keep_on_update(self):
        """
        Not supported yet
        """
        return None

    @property
    def status(self) -> str:
        """
        A string containing one of the following:
        stable: This package is stable for general use.
        testing: This package is in a testing phase,
                 users should be cautious and report issues.
        development: This package is in a development phase
                     and should not be expected to work fully.
        deprecated: This package is no longer maintained.
        """
        if not self.__status:
            self.__status = self.required_str("status")
        return self.__status

    @property
    def kicad_version(self) -> str:
        """
        The minimum required KiCad version for this package
        """
        if not self.__kicad_version:
            self.__kicad_version = self.required_str("kicad_version")
        return self.__kicad_version

    @property
    def kicad_version_max(self) -> str:
        """
        The latest KiCad version this package is compatible with
        """
        if not self.__kicad_version_max:
            if "kicad_version_max" in self.target_config:
                kicad_version_max = self.required_str("kicad_version_max")
            else:
                kicad_version_max = ""
            self.__kicad_version_max = kicad_version_max
        return self.__kicad_version_max

    @property
    def tags(self) -> list[str]:
        """
        The list of tags
        """
        if not self.__tags:
            if "tags" in self.target_config:
                tags: list[str] = self.target_config["tags"]
                if not (
                    isinstance(tags, list)
                    and len(tags)
                    and all(isinstance(item, str) for item in tags)
                ):
                    msg = f"Field `{self._BASE}.tags` must be list of strings"
                    raise TypeError(msg)
            else:
                tags = []
            self.__tags = tags
        return self.__tags

    @property
    def icon(self) -> Path:
        if not self.__icon:
            if "icon" in self.target_config:
                icon = self.target_config["icon"]
                if not isinstance(icon, str):
                    msg = f"Field `{self._BASE}.icon` must be a string"
                    raise TypeError(msg)
                if not Path(icon).is_file():
                    msg = f"Field `{self._BASE}.icon` must point to a file"
                    raise TypeError(msg)
            else:
                msg = f"Field `{self._BASE}.icon` not found"
                raise TypeError(msg)
            self.__icon = Path(icon)
        return self.__icon

    def get_metadata(self) -> dict[str, Any]:
        metadata: dict[str, Any] = {
            "$schema": "https://go.kicad.org/pcm/schemas/v1",
            "name": self.name,
            "description": self.description,
            "description_full": self.description_full,
            "identifier": self.identifier,
            "type": self.type,
            "author": self.author,
            "maintainer": self.maintainer,
            "license": self.license,
            "resources": self.resources,
            "tags": self.tags,
            "versions": [
                {
                    "version": self.builder.metadata.version,
                    "status": self.status,
                    "kicad_version": self.kicad_version,
                    "kicad_version_max": self.kicad_version_max,
                }
            ],
        }
        # remove empty optional fields
        for name in ["maintainer", "resources", "tags"]:
            if not metadata[name]:
                del metadata[name]
        if not metadata["versions"][0]["kicad_version_max"]:
            del metadata["versions"][0]["kicad_version_max"]

        return metadata



import os
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Generator

import pytest


@pytest.fixture(scope="session", autouse=True)
def isolation() -> Generator[Path, None, None]:
    with TemporaryDirectory() as d:
        workdir = Path(d).resolve()
        origin = os.getcwd()
        os.chdir(workdir)
        yield workdir
        os.chdir(origin)



# SPDX-FileCopyrightText: 2023-present adamws <adamws@users.noreply.github.com>
#
# SPDX-License-Identifier: MIT
from hatchling.plugin import hookimpl

from hatch_kicad.build import KicadBuilder


@hookimpl
def hatch_register_builder():
    return KicadBuilder



from __future__ import annotations

import json
import os
import tempfile
import zipfile
from pathlib import Path

import pytest
from hatchling.builders.plugin.interface import BuilderInterface

from hatch_kicad.build import KicadBuilder, get_package_metadata


def merge_dicts(x, y):
    z = x.copy()
    z.update(y)
    return z


def build_config(values):
    return {
        "tool": {
            "hatch": {
                "build": {
                    "targets": {"kicad-package": values},
                },
            },
        },
    }


def test_class() -> None:
    assert issubclass(KicadBuilder, BuilderInterface)


def test_default_versions(isolation):
    builder = KicadBuilder(str(isolation))

    assert builder.get_default_versions() == ["standard"]


@pytest.mark.parametrize(
    "name",
    [
        "name",
        "description",
        "description_full",
        "identifier",
        "status",
        "kicad_version",
    ],
)
class TestRequiredStringOptions:
    def test_option(self, name, isolation):
        builder = KicadBuilder(
            str(isolation), config=build_config({name: "property value"})
        )
        assert getattr(builder.config, name) == "property value"

    def test_option_join(self, name, isolation):
        builder = KicadBuilder(
            str(isolation), config=build_config({name: ["property", " value"]})
        )
        assert getattr(builder.config, name) == "property value"

    def test_option_wrong_type(self, name, isolation):
        builder = KicadBuilder(str(isolation), config=build_config({name: True}))
        with pytest.raises(
            TypeError,
            match=f"Field `tool.hatch.build.targets.kicad-package.{name}` "
            "must be a string or list of strings",
        ):
            _ = getattr(builder.config, name)

    def test_option_missing(self, name, isolation):
        builder = KicadBuilder(str(isolation), config={})
        with pytest.raises(
            TypeError,
            match=f"Field `tool.hatch.build.targets.kicad-package.{name}` not found"
        ):
            _ = getattr(builder.config, name)


def test_type(isolation):
    # type is always equal 'plugin'
    builder = KicadBuilder(str(isolation), config={})
    assert builder.config.type == "plugin"


def test_author(isolation):
    # when author specified by `kicad-package` ignore `project.authors`
    config = merge_dicts(
        {"project": {"authors": [{"email": "foo@domain", "name": "foo"}]}},
        build_config({"author": {"name": "bar", "email": "bar@domain"}}),
    )
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.author == {"name": "bar", "contact": {"email": "bar@domain"}}


def test_author_more_contact(isolation):
    # when author specified by `kicad-package` it can have
    # more contact forms than just email
    config = merge_dicts(
        {"project": {"authors": [{"email": "foo@domain", "name": "foo"}]}},
        build_config(
            {
                "author": {
                    "name": "bar",
                    "email": "bar@domain",
                    "website": "https://bar.site",
                }
            }
        ),
    )
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.author == {
        "name": "bar",
        "contact": {"email": "bar@domain", "website": "https://bar.site"},
    }


def test_author_wrong_type(isolation):
    # author must have name
    builder = KicadBuilder(str(isolation), config=build_config({"author": "bar"}))
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.author` "
        "must be a dictionary",
    ):
        _ = builder.config.author


def test_author_without_name(isolation):
    # author must have name
    config = merge_dicts(
        {"project": {"authors": [{"email": "foo@domain", "name": "foo"}]}},
        build_config({"author": {"email": "bar@domain"}}),
    )
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.author` "
        "must have `name` property",
    ):
        _ = builder.config.author


def test_author_with_only_name(isolation):
    # author with name only should have empty contact information
    config = merge_dicts(
        {"project": {"authors": [{"email": "foo@domain", "name": "foo"}]}},
        build_config({"author": {"name": "bar"}}),
    )
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.author == {"name": "bar", "contact": {}}


def test_author_fallback(isolation):
    # when author not specified by `kicad-package`,
    # try to get first from `project.authors`
    config = {"project": {"authors": [{"email": "foo@domain", "name": "foo"}]}}
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.author == {"name": "foo", "contact": {"email": "foo@domain"}}


def test_author_fallback_email_missing(isolation):
    # when author not specified by `kicad-package`,
    # try to get first from `project.authors`,
    # if it has only name, use empty contact information
    config = {"project": {"authors": [{"name": "foo"}]}}
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.author == {"name": "foo", "contact": {}}


def test_author_fallback_missing(isolation):
    # when author not specified by `kicad-package` and `project.authors`,
    # raise an exception
    config = {"project": {"name": "Plugin", "version": "0.1.0"}}
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.author` not found, "
        "failed to get author from `project.authors` value",
    ):
        _ = builder.config.author


def test_author_fallback_email_only(isolation):
    # when author not specified by `kicad-package`
    # and first author from `project.authors` has email only, raise an exception
    config = {"project": {"authors": [{"email": "foo@domain"}]}}
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.author` not found, "
        "failed to get author from `project.authors` value",
    ):
        _ = builder.config.author


def test_maintainer(isolation):
    # when maintainer specified by `kicad-package` ignore `project.maintainers`
    config = merge_dicts(
        {"project": {"maintainers": [{"email": "foo@domain", "name": "foo"}]}},
        build_config({"maintainer": {"name": "bar", "email": "bar@domain"}}),
    )
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.maintainer == {
        "name": "bar",
        "contact": {"email": "bar@domain"},
    }


def test_maintainer_fallback(isolation):
    # when maintainer not specified by `kicad-package`,
    # try to get first from `project.maintainers`
    config = {"project": {"maintainers": [{"email": "foo@domain", "name": "foo"}]}}
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.maintainer == {
        "name": "foo",
        "contact": {"email": "foo@domain"},
    }


def test_maintainer_fallback_missing(isolation):
    # when maintainer not specified by `kicad-package`
    # and `project.maintainers` missing,
    # return empty dictionary (maintainer is not required)
    config = {"project": {"name": "Plugin", "version": "0.1.0"}}
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.maintainer is None


def test_maintainer_wrong_type(isolation):
    # author must have name
    builder = KicadBuilder(str(isolation), config=build_config({"maintainer": "bar"}))
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.maintainer` "
        "must be a dictionary",
    ):
        _ = builder.config.maintainer


def test_license(isolation):
    config = merge_dicts(
        {"project": {"name": "Plugin", "license": "gpl-3.0"}},
        build_config({"license": "MIT"}),
    )
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.license == "MIT"


def test_license_wrong_type(isolation):
    config = merge_dicts(
        {"project": {"name": "Plugin", "license": "gpl-3.0"}},
        build_config({"license": True}),
    )
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.license` "
        "must be a string or list of strings",
    ):
        _ = builder.config.license


def test_license_fallback(isolation):
    config = {"project": {"name": "Plugin", "license": {"text": "MIT"}}}
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.license == "MIT"


def test_license_fallback_missing(isolation):
    config = {"project": {"name": "Plugin"}}
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(
        TypeError,
        match=(
            "Field `tool.hatch.build.targets.kicad-package.license` not found, "
            "failed to deduce license from `project.license` value.\n"
            'Define `license = {text = "<value>"} in `project` or '
            '`license = "<value>" in `tool.hatch.build.targets.kicad-package'
        ),
    ):
        _ = builder.config.license


def test_resources(isolation):
    config = merge_dicts(
        {"project": {"name": "Plugin", "urls": {"Website": "https://foo.site"}}},
        build_config({"resources": {"Website": "https://bar.site"}}),
    )
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.resources == {"Website": "https://bar.site"}


def test_resources_many(isolation):
    config = merge_dicts(
        {"project": {"name": "Plugin", "urls": {"Website": "https://foo.site"}}},
        build_config(
            {
                "resources": {
                    "Website": "https://bar.site",
                    "Bug Tracker": "https://bugs.bar.site",
                }
            }
        ),
    )
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.resources == {
        "Website": "https://bar.site",
        "Bug Tracker": "https://bugs.bar.site",
    }


def test_resources_wrong_type(isolation):
    config = merge_dicts(
        {"project": {"name": "Plugin", "urls": {"Website": "https://foo.site"}}},
        build_config({"resources": "https://bar.site"}),
    )
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.resources` must be a dict",
    ):
        _ = builder.config.resources


def test_resources_fallback(isolation):
    config = {"project": {"name": "Plugin", "urls": {"Website": "https://foo.site"}}}
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.resources == {"Website": "https://foo.site"}


def test_resources_fallback_missing(isolation):
    config = {"project": {"name": "Plugin"}}
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.resources == {}


def test_keep_on_update(isolation):
    builder = KicadBuilder(str(isolation), config={})
    assert builder.config.keep_on_update is None


def test_kicad_version_max(isolation):
    config = build_config({"kicad_version_max": "6.0"})
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.kicad_version_max == "6.0"


def test_kicad_version_max_wrong_type(isolation):
    config = build_config({"kicad_version_max": True})
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.kicad_version_max` "
        "must be a string or list of strings",
    ):
        _ = builder.config.kicad_version_max


def test_kicad_version_max_missing(isolation):
    builder = KicadBuilder(str(isolation), config={})
    assert builder.config.kicad_version_max == ""


def test_tags(isolation):
    config = build_config({"tags": ["tag1", "tag2"]})
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.tags == ["tag1", "tag2"]


def test_tags_missing(isolation):
    builder = KicadBuilder(str(isolation), config={})
    assert builder.config.tags == []


def test_tags_wrong_type(isolation):
    config = build_config({"tags": [True]})
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.tags` "
        "must be list of strings",
    ):
        _ = builder.config.tags


def test_icon(isolation):
    tf = tempfile.NamedTemporaryFile()
    config = build_config({"icon": tf.name})
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.icon == Path(tf.name)


def test_icon_missing(isolation):
    builder = KicadBuilder(str(isolation), config={})
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.icon` not found"
    ):
        _ = builder.config.icon


def test_icon_does_not_exist(isolation):
    config = build_config({"icon": "src/icon.png"})
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.icon` "
        "must point to a file",
    ):
        _ = builder.config.icon


def test_icon_wrong_type(isolation):
    config = build_config({"icon": ["src/icon.png"]})
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(
        TypeError,
        match="Field `tool.hatch.build.targets.kicad-package.icon` must be a string",
    ):
        _ = builder.config.icon


def test_get_metadata(isolation):
    data = {
        "name": "Plugin Name",
        "description": "Short Decription",
        "description_full": ["Full multiline\n", "description"],
        "identifier": "com.plugin.identifier",
        "author": {"name": "bar", "email": "bar@domain"},
        "license": "MIT",
        "status": "stable",
        "kicad_version": "6.0",
    }
    config = merge_dicts(
        {"project": {"name": "Plugin", "version": "0.0.1"}}, build_config(data)
    )
    builder = KicadBuilder(str(isolation), config=config)
    assert builder.config.get_metadata() == {
        "$schema": "https://go.kicad.org/pcm/schemas/v1",
        "name": "Plugin Name",
        "description": "Short Decription",
        "description_full": "Full multiline\ndescription",
        "identifier": "com.plugin.identifier",
        "type": "plugin",
        "author": {"name": "bar", "contact": {"email": "bar@domain"}},
        "license": "MIT",
        "versions": [{"kicad_version": "6.0", "status": "stable", "version": "0.0.1"}],
    }


def test_package_metadata_calculation(request):
    test_dir = Path(request.module.__file__).parent
    metadata = get_package_metadata(f"{test_dir}/example.zip")
    assert (
        metadata["download_sha256"]
        == "b97d51ed4b8f3efcf53fffbfdc6a353fe516da1efbf807b2750ef3873e8e63ef"
    )
    assert metadata["download_size"] == 174
    assert metadata["install_size"] == 10


def get_zip_contents(zip_path) -> list[str]:
    content = []
    with zipfile.ZipFile(zip_path, "r") as zip_ref:
        for file_name in zip_ref.namelist():
            content.append(file_name)
    return content


def test_build_standard(isolation):
    dist_dir = f"{isolation}/dist"
    src_dir = f"{isolation}/src"
    os.mkdir(dist_dir)
    os.mkdir(src_dir)
    icon = tempfile.NamedTemporaryFile(dir=src_dir)
    sources = [tempfile.NamedTemporaryFile(dir=src_dir, suffix=".py") for _ in range(5)]
    data = {
        "name": "Plugin Name",
        "description": "Short Decription",
        "description_full": ["Full multiline\n", "description"],
        "identifier": "com.plugin.identifier",
        "author": {"name": "bar", "email": "bar@domain"},
        "license": "MIT",
        "status": "stable",
        "kicad_version": "6.0",
        "icon": icon.name,
        "sources": ["src"],
        "include": ["src/*.py"],
    }
    config = merge_dicts(
        {"project": {"name": "Plugin", "version": "0.0.1"}}, build_config(data)
    )
    builder = KicadBuilder(str(isolation), config=config)
    builder.build_standard(dist_dir)
    with open(f"{dist_dir}/metadata.json") as f:
        metadata_result = json.load(f)
        version = metadata_result["versions"][0]
        assert "download_sha256" in version
        assert "download_size" in version
        assert "install_size" in version
        del version["download_sha256"]
        del version["download_size"]
        del version["install_size"]
        assert metadata_result == builder.config.get_metadata()

    in_zip = get_zip_contents(f"{isolation}/dist/Plugin-0.0.1.zip")
    expected = ["resources/icon.png", "metadata.json"]
    for s in sources:
        name = Path(s.name).name
        expected.append(f"plugin/{name}")
    assert len(in_zip) == len(expected) and sorted(in_zip) == sorted(expected)


def test_build_standard_wrong_config(monkeypatch, isolation):
    def mock_abort(*args, **kwargs):
        _ = args, kwargs
        msg = "Abort called"
        raise Exception(msg)
    monkeypatch.setattr("hatchling.bridge.app.Application.abort", mock_abort)
    config = {"project": {"name": "Plugin", "version": "0.1.0"}}
    builder = KicadBuilder(str(isolation), config=config)
    with pytest.raises(Exception, match="Abort called"):
        builder.build_standard(str(isolation))



# SPDX-FileCopyrightText: 2023-present adamws <adamws@users.noreply.github.com>
#
# SPDX-License-Identifier: MIT
from __future__ import annotations

import hashlib
import json
import os
import zipfile
from pathlib import Path
from typing import Any, Callable, TypedDict

from hatchling.builders.plugin.interface import BuilderInterface

from hatch_kicad.config import KicadBuilderConfig

__all__ = ["KicadBuilder"]

READ_SIZE = 65536


class PackageMetadata(TypedDict):
    download_sha256: str
    download_size: int
    install_size: int


def getsha256(filename) -> str:
    sha256 = hashlib.sha256()
    with open(filename, "rb") as f:
        while data := f.read(READ_SIZE):
            sha256.update(data)
    return sha256.hexdigest()


def get_package_metadata(filename) -> PackageMetadata:
    z = zipfile.ZipFile(filename, "r")
    install_size = sum(entry.file_size for entry in z.infolist() if not entry.is_dir())
    return {
        "download_sha256": getsha256(filename),
        "download_size": os.path.getsize(filename),
        "install_size": install_size,
    }


class KicadBuilder(BuilderInterface):
    PLUGIN_NAME = "kicad-package"

    @classmethod
    def get_config_class(cls):
        return KicadBuilderConfig

    def get_version_api(self) -> dict[str, Callable[..., str]]:
        return {"standard": self.build_standard}

    def build_standard(self, directory: str, **build_data: Any) -> str:
        project_name = self.normalize_file_name_component(self.metadata.core.raw_name)
        zip_name = f"{project_name}-{self.metadata.version}.zip"
        zip_target = Path(directory, zip_name)
        metadata_target = Path(directory, "metadata.json")

        try:
            metadata: dict[str, Any] = self.config.get_metadata()
            with open(metadata_target, "w") as f:
                json.dump(metadata, f, indent=4)

            with zipfile.ZipFile(zip_target, "w", zipfile.ZIP_DEFLATED) as zipf:
                for file in self.recurse_included_files():
                    zipf.write(file.path, f"plugin/{file.distribution_path}")
                zipf.write(self.config.icon, "resources/icon.png")
                zipf.write(metadata_target, "metadata.json")

            package_version = metadata["versions"][0]
            package_version.update(get_package_metadata(zip_target))
            self.app.display_info("package details:")
            self.app.display_info(json.dumps(package_version, indent=4))
            # update with calculated metadata
            with open(metadata_target, "w") as f:
                json.dump(metadata, f, indent=4)
        except Exception as e:
            self.app.display_error(str(e))
            self.app.abort("Build failed!")

        return os.fspath(zip_target)
